@@ -13,10 +13,11 @@ import os
 import time
 import zipfile
 import requests
+from jinja2 import Environment, PackageLoader
 
 from transcriptic.english import AutoprotocolParser
 from transcriptic.config import Connection
-from transcriptic.util import iter_json, flatmap, ascii_encode
+from transcriptic.util import iter_json, flatmap, ascii_encode, makedirs
 from transcriptic import routes
 from os.path import isfile
 from collections import OrderedDict
@@ -88,7 +89,7 @@ class HiddenOption(click.Option):
         self.__hidden__ = __hidden__
 
     def get_help_record(self, ctx):
-        """This hijacks the help record so that a hidden option does not show 
+        """This hijacks the help record so that a hidden option does not show
         up in the help text
         """
         if self.__hidden__:
@@ -131,11 +132,11 @@ _CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
 @click.pass_context
 def cli(ctx, api_root, email, token, organization, config):
     """A command line tool for working with Transcriptic.
-    
+
     Note: This is the main entry point of the CLI. If specifying credentials,
     note that the order of preference is: --flag, environment then config file.
-    
-    Example: `transcriptic --organization "my_org" projects` >> 
+
+    Example: `transcriptic --organization "my_org" projects` >>
     `export USER_ORGANIZATION="my_org"` >> `"organization_id": "my_org" in
      ~/.transcriptic
     """
@@ -525,6 +526,45 @@ def delete_package(ctx, name, force):
             click.echo("\n" + str(err))
 
 
+@cli.command()
+@click.pass_context
+@click.argument('name')
+def generate_protocol(ctx, name):
+    """Generate a python protocol scaffold"""
+    # TODO we should update click to be like rails
+    #   transcriptic generate protocol FOO
+    #   transcriptic generate other_type FOO
+    env = Environment(loader=PackageLoader('transcriptic', 'templates'))
+    template_infos = [
+        {"template_name": 'manifest.json.jinja',    "file_name": "manifest.json"},
+        {"template_name": 'protocol.py.jinja',      "file_name": "{}.py".format(name)},
+        {"template_name": 'requirements.txt.jinja', "file_name": "requirements.txt"}
+    ]
+
+    # make directory for protocol
+    dirname = name
+    makedirs(dirname, exist_ok=True)
+
+    # write __init__ package file
+    open('{}/{}'.format(dirname, '__init__.py'), 'w').write('')
+
+    for template_info in template_infos:
+        template_name = template_info["template_name"]
+        file_name     = template_info["file_name"]
+        template      = env.get_template(template_name)
+        file          = open('{}/{}'.format(dirname, file_name), 'w')
+        output        = template.render(name=name)
+        file.write(output)
+
+    click.echo("Successfully generated protocol '{}'!".format(name))
+    click.echo("Testing the protocol is as easy as:")
+    click.echo("")
+    click.echo("\tcd {}".format(name))
+    click.echo("\tpip install -r requirements.txt")
+    click.echo("\ttranscriptic preview {}".format(name))
+    click.echo("\ttranscriptic launch --local {} -p SOME_PROJECT_ID".format(name))
+
+
 @cli.command()
 @click.pass_context
 @click.option("-i")
@@ -935,13 +975,12 @@ def preview(ctx, protocol_name, view, dye_test):
 @cli.command()
 @click.argument('file', default='-')
 @click.pass_context
-@click.option('--tree', '-t', is_flag=True,
-              help='Prints a job tree with instructions as leaf nodes')
-@click.option('--lookup', '-l', is_flag=True,
-              help='Queries Transcriptic to convert resourceID to string')
+@click.option('--html',   '-x', is_flag=True, help='Generates an html view of the autoprotocol')
+@click.option('--tree',   '-t', is_flag=True, help='Prints a job tree with instructions as leaf nodes')
+@click.option('--lookup', '-l', is_flag=True, help='Queries Transcriptic to convert resourceID to string')
 # time allowance is on order of seconds
 @click.option('--runtime', type=click.INT, default=5)
-def summarize(ctx, file, tree, lookup, runtime):
+def summarize(ctx, file, html, tree, lookup, runtime):
     """Summarize Autoprotocol as a list of plain English steps, as well as a
     visualized Job Tree contingent upon desired runtime allowance (in seconds).
     A Job Tree refers to a structure of protocol based on container dependency,
@@ -955,27 +994,35 @@ def summarize(ctx, file, tree, lookup, runtime):
     python my_script.py | transcriptic summarize --tree
 
     python my_script.py | transcriptic summarize --tree --runtime 20
+
+    python my_script.py | transcriptic summarize --html
     """
     with click.open_file(file, 'r') as f:
         try:
             protocol = json.loads(f.read())
         except ValueError:
-            click.echo(
-                "The autoprotocol you're trying to summarize is invalid.")
+            click.echo("The autoprotocol you're trying to summarize is invalid.")
             return
 
-    if lookup:
+    # context to pass into the parser given that the default ctx is not set for this method.
+    api_context = None
+
+    if lookup or html:
         try:
-            config = '~/.transcriptic'
-            ctx.obj = ContextObject()
+            config      = '~/.transcriptic'
+            ctx.obj     = ContextObject()
             ctx.obj.api = Connection.from_file(config)
-            parser = AutoprotocolParser(protocol, ctx=ctx)
+            api_context = ctx
         except:
             click.echo("Connection with Transcriptic failed. "
                        "Summarizing without lookup.", err=True)
-            parser = AutoprotocolParser(protocol)
-    else:
-        parser = AutoprotocolParser(protocol)
+
+    if html:
+        url = ProtocolPreview(protocol, ctx.obj.api).preview_url
+        click.echo("View your protocol here {}".format(url))
+        return
+
+    parser = AutoprotocolParser(protocol, ctx=api_context)
 
     if tree:
         import multiprocessing
@@ -1186,12 +1233,12 @@ def launch(ctx, protocol, project, save_input, local, accept_quote, params,
         else:
             """
             In the case of a local `launch`, we need to generate `inputs` from
-            `raw_inputs`, since the `run_protocol` function takes in JSON which 
+            `raw_inputs`, since the `run_protocol` function takes in JSON which
             is `inputs`-formatted.
             `inputs` is basically an extended version of `raw_inputs`, where
             we populate properties such as aliquot information for specified
             containerIds.
-            In order to generate these `inputs`, we can create a new quick 
+            In order to generate these `inputs`, we can create a new quick
             launch
             """
             try:
@@ -1287,11 +1334,20 @@ def _get_quick_launch(ctx, protocol, project):
     return quick_launch
 
 @cli.command()
+@click.argument(
+    'organization',
+    metavar='ORGANIZATION_NAME',
+    type=str,
+    required=False
+)
 @click.pass_context
-def select_org(ctx):
-    """Allows you to switch organizations"""
+def select_org(ctx, organization=None):
+    """Allows you to switch organizations. If the organization argument
+    is provided, this will directly select the specified organization.
+    """
     org_list = [{"name": org['name'], "subdomain": org['subdomain']} for org in ctx.obj.api.organizations()]
-    organization = org_prompt(org_list)
+    if organization is None:
+        organization = org_prompt(org_list)
 
     r = ctx.obj.api.get_organization(org_id=organization)
     if r.status_code != 200:
@@ -1600,8 +1656,9 @@ class ProtocolPreview(object):
     An object for previewing protocols
     """
     def __init__(self, protocol, connection):
-        self.protocol = protocol
-        self.preview_url = connection.preview_protocol(protocol)
+        self.protocol    = protocol
+        preview_id       = connection.preview_protocol(protocol)
+        self.preview_url = connection.get_route('preview_protocol_embed', preview_id=preview_id)
 
     def _repr_html_(self):
         return """<iframe src="%s" frameborder="0" allowtransparency="true" \
